@@ -46,38 +46,40 @@ type Exporter struct {
 }
 
 type Options struct {
-	User                  string
-	Password              string
-	Namespace             string
-	PasswordMap           map[string]string
-	ConfigCommandName     string
-	CheckKeys             string
-	CheckSingleKeys       string
-	CheckStreams          string
-	CheckSingleStreams    string
-	CheckKeysBatchSize    int64
-	CheckKeyGroups        string
-	MaxDistinctKeyGroups  int64
-	CountKeys             string
-	LuaScript             []byte
-	ClientCertFile        string
-	ClientKeyFile         string
-	CaCertFile            string
-	InclConfigMetrics     bool
-	RedactConfigMetrics   bool
-	InclSystemMetrics     bool
-	SkipTLSVerification   bool
-	SetClientName         bool
-	IsTile38              bool
-	IsCluster             bool
-	ExportClientList      bool
-	ExportClientsInclPort bool
-	ConnectionTimeouts    time.Duration
-	MetricsPath           string
-	RedisMetricsOnly      bool
-	PingOnConnect         bool
-	Registry              *prometheus.Registry
-	BuildInfo             BuildInfo
+	User                      string
+	Password                  string
+	Namespace                 string
+	PasswordMap               map[string]string
+	ConfigCommandName         string
+	CheckKeys                 string
+	CheckSingleKeys           string
+	CheckStreams              string
+	CheckSingleStreams        string
+	CheckKeysBatchSize        int64
+	CheckKeyGroups            string
+	MaxDistinctKeyGroups      int64
+	CountKeys                 string
+	LuaScript                 map[string][]byte
+	ClientCertFile            string
+	ClientKeyFile             string
+	CaCertFile                string
+	InclConfigMetrics         bool
+	DisableExportingKeyValues bool
+	RedactConfigMetrics       bool
+	InclSystemMetrics         bool
+	SkipTLSVerification       bool
+	SetClientName             bool
+	IsTile38                  bool
+	IsCluster                 bool
+	ExportClientList          bool
+	ExportClientsInclPort     bool
+	ConnectionTimeouts        time.Duration
+	MetricsPath               string
+	RedisMetricsOnly          bool
+	PingOnConnect             bool
+	RedisPwdFile              string
+	Registry                  *prometheus.Registry
+	BuildInfo                 BuildInfo
 }
 
 // NewRedisExporter returns a new exporter of Redis metrics.
@@ -334,11 +336,6 @@ func NewRedisExporter(redisURI string, opts Options) (*Exporter, error) {
 
 	e.metricDescriptions = map[string]*prometheus.Desc{}
 
-	connectedClientsLabels := []string{"name", "created_at", "idle_since", "flags", "db", "omem", "cmd", "host"}
-	if e.options.ExportClientsInclPort {
-		connectedClientsLabels = append(connectedClientsLabels, "port")
-	}
-
 	for k, desc := range map[string]struct {
 		txt  string
 		lbls []string
@@ -347,10 +344,10 @@ func NewRedisExporter(redisURI string, opts Options) (*Exporter, error) {
 		"commands_failed_calls_total":                  {txt: `Total number of errors prior command execution per command`, lbls: []string{"cmd"}},
 		"commands_rejected_calls_total":                {txt: `Total number of errors within command execution per command`, lbls: []string{"cmd"}},
 		"commands_total":                               {txt: `Total number of calls per command`, lbls: []string{"cmd"}},
+		"commands_latencies_usec":                      {txt: `A histogram of latencies per command`, lbls: []string{"cmd"}},
 		"latency_percentiles_usec":                     {txt: `A summary of latency percentile distribution per command`, lbls: []string{"cmd"}},
 		"config_key_value":                             {txt: `Config key and value`, lbls: []string{"key", "value"}},
 		"config_value":                                 {txt: `Config key and value as metric`, lbls: []string{"key"}},
-		"connected_clients_details":                    {txt: "Details about connected clients", lbls: connectedClientsLabels},
 		"connected_slave_lag_seconds":                  {txt: "Lag of connected slave", lbls: []string{"slave_ip", "slave_port", "slave_state"}},
 		"connected_slave_offset_bytes":                 {txt: "Offset of connected slave", lbls: []string{"slave_ip", "slave_port", "slave_state"}},
 		"db_avg_ttl_seconds":                           {txt: "Avg TTL in seconds", lbls: []string{"db"}},
@@ -374,7 +371,8 @@ func NewRedisExporter(redisURI string, opts Options) (*Exporter, error) {
 		"master_link_up":                               {txt: "Master link status on Redis slave", lbls: []string{"master_host", "master_port"}},
 		"master_sync_in_progress":                      {txt: "Master sync in progress", lbls: []string{"master_host", "master_port"}},
 		"number_of_distinct_key_groups":                {txt: `Number of distinct key groups`, lbls: []string{"db"}},
-		"script_values":                                {txt: "Values returned by the collect script", lbls: []string{"key"}},
+		"script_result":                                {txt: "Result of the collect script evaluation", lbls: []string{"filename"}},
+		"script_values":                                {txt: "Values returned by the collect script", lbls: []string{"key", "filename"}},
 		"sentinel_master_ok_sentinels":                 {txt: "The number of okay sentinels monitoring this master", lbls: []string{"master_name", "master_address"}},
 		"sentinel_master_ok_slaves":                    {txt: "The number of okay slaves of the master", lbls: []string{"master_name", "master_address"}},
 		"sentinel_master_sentinels":                    {txt: "The number of sentinels monitoring this master", lbls: []string{"master_name", "master_address"}},
@@ -394,11 +392,16 @@ func NewRedisExporter(redisURI string, opts Options) (*Exporter, error) {
 		"stream_group_consumer_idle_seconds":           {txt: `Consumer idle time in seconds`, lbls: []string{"db", "stream", "group", "consumer"}},
 		"stream_group_consumer_messages_pending":       {txt: `Pending number of messages for this specific consumer`, lbls: []string{"db", "stream", "group", "consumer"}},
 		"stream_group_consumers":                       {txt: `Consumers count of stream group`, lbls: []string{"db", "stream", "group"}},
+		"stream_group_entries_read":                    {txt: `Total number of entries read from the stream group`, lbls: []string{"db", "stream", "group"}},
+		"stream_group_lag":                             {txt: `The number of messages waiting to be delivered to the stream group's consumers`, lbls: []string{"db", "stream", "group"}},
 		"stream_group_last_delivered_id":               {txt: `The epoch timestamp (ms) of the last delivered message`, lbls: []string{"db", "stream", "group"}},
 		"stream_group_messages_pending":                {txt: `Pending number of messages in that stream group`, lbls: []string{"db", "stream", "group"}},
 		"stream_groups":                                {txt: `Groups count of stream`, lbls: []string{"db", "stream"}},
 		"stream_last_generated_id":                     {txt: `The epoch timestamp (ms) of the latest message on the stream`, lbls: []string{"db", "stream"}},
 		"stream_length":                                {txt: `The number of elements of the stream`, lbls: []string{"db", "stream"}},
+		"stream_max_deleted_entry_id":                  {txt: `The epoch timestamp (ms) of last message was deleted from the stream`, lbls: []string{"db", "stream"}},
+		"stream_first_entry_id":                        {txt: `The epoch timestamp (ms) of the first message in the stream`, lbls: []string{"db", "stream"}},
+		"stream_last_entry_id":                         {txt: `The epoch timestamp (ms) of the last message in the stream`, lbls: []string{"db", "stream"}},
 		"stream_radix_tree_keys":                       {txt: `Radix tree keys count"`, lbls: []string{"db", "stream"}},
 		"stream_radix_tree_nodes":                      {txt: `Radix tree nodes count`, lbls: []string{"db", "stream"}},
 		"up":                                           {txt: "Information about the Redis instance"},
@@ -432,6 +435,7 @@ func NewRedisExporter(redisURI string, opts Options) (*Exporter, error) {
 	e.mux.HandleFunc("/", e.indexHandler)
 	e.mux.HandleFunc("/scrape", e.scrapeHandler)
 	e.mux.HandleFunc("/health", e.healthHandler)
+	e.mux.HandleFunc("/-/reload", e.reloadPwdFile)
 
 	return e, nil
 }
@@ -483,14 +487,23 @@ func (e *Exporter) Collect(ch chan<- prometheus.Metric) {
 	ch <- e.targetScrapeRequestErrors
 }
 
-func (e *Exporter) extractConfigMetrics(ch chan<- prometheus.Metric, config []string) (dbCount int, err error) {
+func (e *Exporter) extractConfigMetrics(ch chan<- prometheus.Metric, config []interface{}) (dbCount int, err error) {
 	if len(config)%2 != 0 {
 		return 0, fmt.Errorf("invalid config: %#v", config)
 	}
 
 	for pos := 0; pos < len(config)/2; pos++ {
-		strKey := config[pos*2]
-		strVal := config[pos*2+1]
+		strKey, err := redis.String(config[pos*2], nil)
+		if err != nil {
+			log.Errorf("invalid config key name, err: %s, skipped", err)
+			continue
+		}
+
+		strVal, err := redis.String(config[pos*2+1], nil)
+		if err != nil {
+			log.Errorf("invalid config value for key name %s, err: %s, skipped", strKey, err)
+			continue
+		}
 
 		if strKey == "databases" {
 			if dbCount, err = strconv.Atoi(strVal); err != nil {
@@ -570,7 +583,7 @@ func (e *Exporter) scrapeRedisHost(ch chan<- prometheus.Metric) error {
 	}
 
 	dbCount := 0
-	if config, err := redis.Strings(doRedisCmd(c, e.options.ConfigCommandName, "GET", "*")); err == nil {
+	if config, err := redis.Values(doRedisCmd(c, e.options.ConfigCommandName, "GET", "*")); err == nil {
 		log.Debugf("Redis CONFIG GET * result: [%#v]", config)
 		dbCount, err = e.extractConfigMetrics(ch, config)
 		if err != nil {
@@ -612,7 +625,7 @@ func (e *Exporter) scrapeRedisHost(ch chan<- prometheus.Metric) error {
 
 	e.extractInfoMetrics(ch, infoAll, dbCount)
 
-	e.extractLatencyMetrics(ch, c)
+	e.extractLatencyMetrics(ch, infoAll, c)
 
 	if e.options.IsCluster {
 		clusterClient, err := e.connectToRedisCluster()
@@ -648,8 +661,10 @@ func (e *Exporter) scrapeRedisHost(ch chan<- prometheus.Metric) error {
 	}
 
 	if len(e.options.LuaScript) > 0 {
-		if err := e.extractLuaScriptMetrics(ch, c); err != nil {
-			return err
+		for filename, script := range e.options.LuaScript {
+			if err := e.extractLuaScriptMetrics(ch, c, filename, script); err != nil {
+				return err
+			}
 		}
 	}
 
