@@ -12,12 +12,15 @@ import (
 // All fields of the streamInfo struct must be exported
 // because of redis.ScanStruct (reflect) limitations
 type streamInfo struct {
-	Length           int64  `redis:"length"`
-	RadixTreeKeys    int64  `redis:"radix-tree-keys"`
-	RadixTreeNodes   int64  `redis:"radix-tree-nodes"`
-	LastGeneratedId  string `redis:"last-generated-id"`
-	Groups           int64  `redis:"groups"`
-	StreamGroupsInfo []streamGroupsInfo
+	Length            int64  `redis:"length"`
+	RadixTreeKeys     int64  `redis:"radix-tree-keys"`
+	RadixTreeNodes    int64  `redis:"radix-tree-nodes"`
+	LastGeneratedId   string `redis:"last-generated-id"`
+	Groups            int64  `redis:"groups"`
+	MaxDeletedEntryId string `redis:"max-deleted-entry-id"`
+	FirstEntryId      string
+	LastEntryId       string
+	StreamGroupsInfo  []streamGroupsInfo
 }
 
 type streamGroupsInfo struct {
@@ -25,6 +28,8 @@ type streamGroupsInfo struct {
 	Consumers                int64  `redis:"consumers"`
 	Pending                  int64  `redis:"pending"`
 	LastDeliveredId          string `redis:"last-delivered-id"`
+	EntriesRead              int64  `redis:"entries-read"`
+	Lag                      int64  `redis:"lag"`
 	StreamGroupConsumersInfo []streamGroupConsumersInfo
 }
 
@@ -39,12 +44,17 @@ func getStreamInfo(c redis.Conn, key string) (*streamInfo, error) {
 	if err != nil {
 		return nil, err
 	}
+
 	// Scan slice to struct
 	var stream streamInfo
 	if err := redis.ScanStruct(v, &stream); err != nil {
 		return nil, err
 	}
 
+	// Extract first and last id from slice
+	stream.FirstEntryId = getEntryId(v, 17)
+	stream.LastEntryId = getEntryId(v, 19)
+
 	stream.StreamGroupsInfo, err = scanStreamGroups(c, key)
 	if err != nil {
 		return nil, err
@@ -54,6 +64,20 @@ func getStreamInfo(c redis.Conn, key string) (*streamInfo, error) {
 	return &stream, nil
 }
 
+func getEntryId(redisValue []interface{}, index int) string {
+	var emptyStreamId = ""
+
+	if len(redisValue) < index || len(redisValue) > index && len(redisValue[index].([]interface{})) < 2 {
+		return emptyStreamId
+	}
+
+	entryId, ok := redisValue[index].([]interface{})[0].([]byte)
+	if !ok {
+		return emptyStreamId
+	}
+	return string(entryId)
+}
+
 func scanStreamGroups(c redis.Conn, stream string) ([]streamGroupsInfo, error) {
 	groups, err := redis.Values(doRedisCmd(c, "XINFO", "GROUPS", stream))
 	if err != nil {
@@ -163,11 +187,16 @@ func (e *Exporter) extractStreamMetrics(ch chan<- prometheus.Metric, c redis.Con
 		e.registerConstMetricGauge(ch, "stream_radix_tree_nodes", float64(info.RadixTreeNodes), dbLabel, k.key)
 		e.registerConstMetricGauge(ch, "stream_last_generated_id", parseStreamItemId(info.LastGeneratedId), dbLabel, k.key)
 		e.registerConstMetricGauge(ch, "stream_groups", float64(info.Groups), dbLabel, k.key)
+		e.registerConstMetricGauge(ch, "stream_max_deleted_entry_id", parseStreamItemId(info.MaxDeletedEntryId), dbLabel, k.key)
+		e.registerConstMetricGauge(ch, "stream_first_entry_id", parseStreamItemId(info.FirstEntryId), dbLabel, k.key)
+		e.registerConstMetricGauge(ch, "stream_last_entry_id", parseStreamItemId(info.LastEntryId), dbLabel, k.key)
 
 		for _, g := range info.StreamGroupsInfo {
 			e.registerConstMetricGauge(ch, "stream_group_consumers", float64(g.Consumers), dbLabel, k.key, g.Name)
 			e.registerConstMetricGauge(ch, "stream_group_messages_pending", float64(g.Pending), dbLabel, k.key, g.Name)
 			e.registerConstMetricGauge(ch, "stream_group_last_delivered_id", parseStreamItemId(g.LastDeliveredId), dbLabel, k.key, g.Name)
+			e.registerConstMetricGauge(ch, "stream_group_entries_read", float64(g.EntriesRead), dbLabel, k.key, g.Name)
+			e.registerConstMetricGauge(ch, "stream_group_lag", float64(g.Lag), dbLabel, k.key, g.Name)
 			for _, c := range g.StreamGroupConsumersInfo {
 				e.registerConstMetricGauge(ch, "stream_group_consumer_messages_pending", float64(c.Pending), dbLabel, k.key, g.Name, c.Name)
 				e.registerConstMetricGauge(ch, "stream_group_consumer_idle_seconds", float64(c.Idle)/1e3, dbLabel, k.key, g.Name, c.Name)
